/***************************************************************************
 *   Copyright (C) 2010 by Giacomo Strangolino,T2 PT 025,,   *
 *   giacomo.strangolino@elettra.trieste.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "ftfconfig.h"
#include <qtango.h>

#include <QTWatcher>
#include <ReadersGroup>
#include "weightmanager.h"
#include "optsensorsweightmanager.h"

ftfconfig::ftfconfig(QWidget *parent) : QWidget(parent)
{
	ui.setupUi(this);
    ui.comboFileList->setPeriod(1000);
	ui.comboFileList->setSource("$1->GetRmFileList");

    ui.tTextBrowser->setSource("$1->GetErrLog(&tsbNumEvents)");

    WeightManager *weightManager = new WeightManager(ui.tabWidget->widget(1));
    ui.scrollArea->setWidget(weightManager);

    OptSensorsWeightManager *optSensorsWeightManager = new OptSensorsWeightManager(ui.tabWidget->widget(6));
    ui.scrollAreaOptimizationSensors->setWidget(optSensorsWeightManager);
    connect(ui.pbSetOptSensorsAllZero, SIGNAL(clicked()), optSensorsWeightManager, SLOT(setAllToValue()));
    connect(ui.pbSetOptSensorsAllOne, SIGNAL(clicked()), optSensorsWeightManager, SLOT(setAllToValue()));

    {
    QTWatcher *bpmLabelsW = new QTWatcher(this);
    connect(bpmLabelsW, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configureBpmWeightManager(const TangoConfigurationParameters *)));
    connect(bpmLabelsW, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configureOptSensorsWeightManager(const TangoConfigurationParameters *)));
    bpmLabelsW->setAutoDestroy(true); /* auto destroy after first reading */
    bpmLabelsW->setSource("$1->GetBpmNames");
    }

    {
    QTWatcher *corrLabelsW = new QTWatcher(this);
    connect(corrLabelsW, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configureCorrWeightManager(const TangoConfigurationParameters *)));
    corrLabelsW->setAutoDestroy(true); /* auto destroy after first reading */
    corrLabelsW->setSource("$1->GetCorNames");
    }





    {
    QTWatcher *bpmLabelsW = new QTWatcher(this);
    connect(bpmLabelsW, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configureBpmWeightManager(const TangoConfigurationParameters *)));
    bpmLabelsW->setAutoDestroy(true); /* auto destroy after first reading */
    bpmLabelsW->setSource("$1->GetBpmNames");
    }

    {
    QTWatcher *corrLabelsW = new QTWatcher(this);
    connect(corrLabelsW, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configureCorrWeightManager(const TangoConfigurationParameters *)));
    corrLabelsW->setAutoDestroy(true); /* auto destroy after first reading */
    corrLabelsW->setSource("$1->GetCorNames");
    }

    ReadersGroup *pidParRg = new ReadersGroup(this);
    pidParRg->addReader(ui.tlkp, 0);
    pidParRg->addReader(ui.tlki, 1);
    pidParRg->addReader(ui.tlkd, 2);
    pidParRg->setSource("$1/PIDParam");

    ReadersGroup *lowPassParRg = new ReadersGroup(this);
    lowPassParRg->addReader(ui.tlb0, 0);
    lowPassParRg->addReader(ui.tlb1, 1);
    lowPassParRg->addReader(ui.tlb2, 2);
    lowPassParRg->addReader(ui.tla0, 3);
    lowPassParRg->addReader(ui.tla1, 4);
    lowPassParRg->addReader(ui.tla2, 5);

    lowPassParRg->setSource("$1/LowPassParam");

    ui.tpbSetReferenceZero->setVisible(qApp->arguments().contains("--laser"));

    QTimer::singleShot(599, this, SLOT(configOptimizeTargets()));

}

void ftfconfig::configOptimizeTargets()
{
    std::vector<std::string> list;
    Tango::Database *db = new Tango::Database();
    Tango::DbData db_data;
    db_data.push_back(Tango::DbDatum("Targets"));
    try{
        db->get_property("Optimization", db_data);
        db_data[0] >> list;
        ui.cbOptimizeQuick->clear();
        ui.cbOptimizeQuick->insertItem(0, "---");
        for(unsigned int i = 0; i < list.size(); i++)
        {

                ui.cbOptimizeQuick->insertItem(i + 1, QString::fromStdString(list.at(i)));

        }
        connect(ui.cbOptimizeQuick, SIGNAL(currentIndexChanged(int)), this, SLOT(optimizeQuickSelectIndexChanged(int)));
    }
    catch(Tango::DevFailed& e)
    {
        TLog log(e);
        perr("TrajectoryScene::configMaxMin: error getting DeviceList from TrajectoryScene free property");
        perr("%s", qstoc(log.formatMessage()));
    }
}

void ftfconfig::optimizeQuickSelectIndexChanged(int index)
{
    if(index == 0)
        ui.tleOptimizeTargets->setText("");
    else if(ui.cbOptimizeQuick->currentText().contains(";"))
    {
        QString optS;
        int i;
        QStringList pts = ui.cbOptimizeQuick->currentText().split(";");
        for(i = 1; i < pts.size() - 1; i++)
            optS += pts.at(i) + ";";
        optS += pts.at(i);
        ui.tleOptimizeTargets->setText(optS);
    }
}

ftfconfig::~ftfconfig()
{
}    

void ftfconfig::configureBpmWeightManager(const TangoConfigurationParameters *p)
{
    TVariant v = p->currentValue();
    findChild<WeightManager *>()->setupBpms(v.toStringVector().toList(), "$1/WeightTrajHor", "$1/WeightTrajVer");
}

void ftfconfig::configureCorrWeightManager(const TangoConfigurationParameters *p)
{
    TVariant v = p->currentValue();
    findChild<WeightManager *>()->setupCorrAndEig(v.toStringVector().toList(),
                                                  "$1/WeightCorHor", "$1/WeightCorVer",
                                                  "$1/WeightEigHor", "$1/WeightEigVer");
}

void ftfconfig::configureOptSensorsWeightManager(const TangoConfigurationParameters *p)
{
    TVariant v = p->currentValue();
    findChild<OptSensorsWeightManager *>()->setupBpms(v.toStringVector().toList(), "$1/OptimizeBpm");
}
