/***************************************************************************
 *   Copyright (C) 2010 by Giacomo Strangolino,T2 PT 025,,   *
 *   giacomo.strangolino@elettra.trieste.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "ftfconfig.h"
/* the following two are used to register the application to the window manager */
#include <X11/Xlib.h>
#include <QX11Info>

#include <elettracolors.h> /* to set elettra/booster/fermi palette */
#include <tutil.h>  /* for setLoggingTarget() */

#include <TApplication>
#include <TSplashScreen>
// #include <EStyleLoader> /* see comment below */

#define CVSVERSION "$Name:  $"

int main( int argc, char ** argv ) {

    /* introduced in version 3.1. To retrieve the version:
	 * 1. on the executable, execute `strings' and grep with "\$Name"
	 * 2. call qApp->applicationVersion
	 * 3. TApplication exports "version" on the DBus system.
	 */
	const char *cvs_version = CVSVERSION;
	/* Create TApplication */
    TApplication a( argc, argv );
	/* make the version available through QApplication::instance()->applicationVersion() 
	 * and through the DBus message bus.
	 */
	a.setApplicationVersion(cvs_version);
    /* change "fermi" into "booster" or "elettra" to set booster/elettra palettes */
    a.setPalette(EPalette("fermi"));
	
	a.setApplicationName("ftfconfig");
	
	/* comment the following properties if you are not interested in them.
	 * They are exported through QApplication and are available from any
	 * QTango widget by right clicking and executing the "Info" action
	 * of the menu.
	 */
	a.setProperty("author", "Giacomo");
    a.setProperty("mail", "giacomo.strangolino@elettra.trieste.it");
    a.setProperty("phone", "375-8073");
    a.setProperty("office", "T2PT025");
	a.setProperty("hwReferent", "Giulio"); /* name of the referent that provides the device server */
	
    TUtil::instance()->setLoggingTarget(argv[0]);
	
	/* load the style sheet for QTango application. 
	 * The file is read from the "${INSTALL_ROOT}/share/qtango/stylesheets/qtango.css"
	 * file. You can load a custom stylesheet by exporting the environment variable 
	 * "QTANGO_STYLESHEET" and setting it to a different path.
	 * Moreover, you can construct EStyleLoader with a file name as argument, to ignore
	 * the installed "${INSTALL_ROOT}/share/qtango/stylesheets/qtango.css" and the value
	 * of the environment variable.
	 *
	 * This does not work as expected in Qt version 4.4. Uncomment the following two lines
	 * when solved.
	 */
//    EStyleLoader sLoader;
//    a.setStyleSheet(sLoader.styleSheet());

	/* splash screen */
    TSplashScreen splash;

    ftfconfig mw;
    mw.setWindowTitle("ftfconfig");
    if (argc > 1)
           mw.setWindowTitle(mw.windowTitle() + " - " + QApplication::arguments()[1].split('/').last());
    mw.show();
	/* hide splash screen */
    splash.finish(&mw);

    /* register to window manager */
    Display *disp = QX11Info::display();
    Window root_win = (Window) mw.winId();
    XSetCommand(disp, root_win, argv, argc);
	
    return a.exec();
}
