#include "optsensorsweightmanager.h"
#include <QWidget>
#include <QGridLayout>
#include <QLabel>
#include <ENumeric>
#include <TSimpleLabel>
#include <ReadersGroup>
#include <SpectrumWriter>
#include <QPushButton>
#include <QTWatcher>
#include <QTWriter>
#include <macros.h>
#include <eled.h>
#include <QtDebug>
#include <QCoreApplication>
#include "weightlabel.h"

OptSensorsWeightManager::OptSensorsWeightManager(QWidget *containerWidget) : QWidget(containerWidget)
{
    setObjectName("OptSensorsWeightManager");
    m_tabWidget = containerWidget;
    m_currentSelectedIndex = -1;
    connect(m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply"), SIGNAL(clicked()), this, SLOT(apply()));
    m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setEnabled(false);
    m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->setEnabled(false);
    QGridLayout *gridLo = new QGridLayout(this);
}

void OptSensorsWeightManager::setupBpms(const QStringList& labels, const QString &attrName)
{
    QGridLayout *lo = this->findChild<QGridLayout *>();
    QLabel *bpmLabel = new QLabel(this);
    bpmLabel->setText("Bpms");
    bpmLabel->setAlignment(Qt::AlignHCenter);

    QLabel *labelStReadback = new QLabel(this);
    labelStReadback->setText("Status Readback");
    labelStReadback->setAlignment(Qt::AlignHCenter);

    QLabel *labelOptControlled = new QLabel(this);
    labelOptControlled->setText("Optimization Controlled");
    labelOptControlled->setAlignment(Qt::AlignRight);

    lo->addWidget(bpmLabel, 0, 1, 1, 1);
    lo->addWidget(labelStReadback, 0, 2, 1, 1);
    lo->addWidget(labelOptControlled, 0, 3,1 ,1);

    int row = 0;
    {
        ReadersGroup *rgHor = new ReadersGroup(this);
        foreach (QString label, labels)
        {
            QLabel *lab = new QLabel(this);
            lab->setText(label);
            WeightLabel *wl = new WeightLabel(this, label, row);
            wl->setProperty("associatedTangoPoint", attrName);
            wl->setObjectName("Sensor_" + label);
            connect(wl, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgHor->addReader(wl, row);
            
            ELed *ledStReadback = new ELed(this);
            ledStReadback->setObjectName(QString("lsr%1").arg(row));
            ledStReadback->setProperty("index", row);
            ledStReadback->setProperty("source", "");
            
            ELed *ledOptControlled = new ELed(this);
            ledOptControlled->setObjectName(QString("loc%1").arg(row));
            ledOptControlled->setProperty("index", row);
            ledOptControlled->setProperty("source", "");

            QTWatcher *ledsWatcher = new QTWatcher(this);
            connect(ledsWatcher->qtangoComHandle(), SIGNAL(newData(const TVariant&)), this, SLOT(refreshLeds(const TVariant&)));
            ledsWatcher->qtangoComHandle()->setProperty("index", row);
            ledsWatcher->setSource(qApp->arguments().at(1) + QString("->ScanGetParameters(%1)").arg(row));
            connect(ledsWatcher, SIGNAL(readOk(bool)), this, SLOT(ledsWatcherReadFailed(bool)));


            row++;
            lo->addWidget(lab, row, BpmLabels, 1, 1);
            lo->addWidget(wl, row, Bpm, 1, 1);
            lo->addWidget(ledStReadback, row, LedStReadback, 1, 1);
            lo->addWidget(ledOptControlled, row,LedOptimizationControlled, 1, 1);



        }
        rgHor->setSource(attrName);
    }
}

void OptSensorsWeightManager::setAllToValue()
{
    QString val = sender()->property("value").toString();
    m_currentSetPointVector.clear();
    int i = 0;
    foreach(WeightLabel *l, findChildren<WeightLabel *>())
    {
        l->setSelected(false);
        l->setText(val);
        m_currentSetPointVector.append(val);
    }
    QTWriter *writer = findChild<QTWriter *>();
    if(writer)
        writer->QTangoComProxyWriter::execute(m_currentSetPointVector);

}

void OptSensorsWeightManager::manageSelection(const QString &oName)
{
    m_currentSetPointVector.clear();
    m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setEnabled(false);
    m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->setEnabled(false);

    foreach(WeightLabel *l, findChildren<WeightLabel *>())
    {
        l->setSelected(l->objectName() == oName);
        if(l->objectName() == oName)
            m_currentSelectedIndex = l->index();
    }
    /* label */
    m_tabWidget->findChild<TSimpleLabel *>("tslCurrentOptSensorsWeight")->setText(oName);
    QTWatcher *setPtWatcher = new QTWatcher(this);
    /* destroy watcher after first reading (qtango 4.3) */
    setPtWatcher->setAutoDestroy(true);
    connect(setPtWatcher, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configurationReady(const TangoConfigurationParameters*)));

    connect(setPtWatcher->qtangoComHandle(), SIGNAL(newData(TVariant)), this, SLOT(initWriter(TVariant)));
    setPtWatcher->setSource(sender()->property("associatedTangoPoint").toString());
    m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->setProperty("associatedTangoPoint",
                    sender()->property("associatedTangoPoint").toString());
}

void OptSensorsWeightManager::apply()
{
    QTWriter *writer = findChild<QTWriter *>();
    if(writer && m_currentSetPointVector.size() > m_currentSelectedIndex)
    {

        m_currentSetPointVector[m_currentSelectedIndex] = m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->value();
        QVariant argument(m_currentSetPointVector);
        qDebug() << __FUNCTION__ << m_currentSetPointVector << "execute" << writer->realTargets() <<
                    "index " << m_currentSelectedIndex;
        writer->QTangoComProxyWriter::execute(argument);
    }
}

void OptSensorsWeightManager::initWriter(const TVariant &v)
{
    if(m_currentSelectedIndex > -1)
    {
        /* false: get set point */
        QVector<double> setPtValues = v.toDoubleVector(false);
        QVector<double> readValues = v.toDoubleVector(true);
//       printf("\e[1;33msetPtValues sz %d readVal size %d curr index %d\e[0m\n",
//              setPtValues.size(), readValues.size(), m_currentSelectedIndex);
        if(setPtValues.size() == readValues.size() && setPtValues.size() > m_currentSelectedIndex)
        {
            m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setValue(setPtValues.at(m_currentSelectedIndex));
            foreach(double d, setPtValues)
                m_currentSetPointVector << d;
        }
        else
        {
            foreach(double d, readValues)
                m_currentSetPointVector << d;
            qDebug() << "uso read" << m_currentSetPointVector << v.tangoPoint();
            m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setValue(readValues.at(m_currentSelectedIndex));

        }
        QString currentTarget = m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->property("associatedTangoPoint").toString();
        QTWriter *writer = findChild<QTWriter *>();

        if((writer != NULL && writer->targets() != currentTarget) || writer == NULL)
        {
            delete writer;
            writer = new QTWriter(this);
            connect(writer, SIGNAL(connectionOk(bool)), m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply"),
                    SLOT(setEnabled(bool)));
            writer->setTargets(currentTarget);
        }
        if(writer && writer->qtangoComHandle()->configured())
        {
            m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->setEnabled(true);
            m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->
                    setToolTip(QString("%1").arg(currentTarget.split("/").last()));
        }
    }
}

void OptSensorsWeightManager::configurationReady(const TangoConfigurationParameters *cp)
{
    AttributeInfoEx aiex = cp->attributeInfoEx();
    m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setEnabled(true);
    QString tip = QString("%1\n%2").arg(QString::fromStdString(aiex.name)).arg(
                QString::fromStdString(aiex.description));
    m_tabWidget->findChild<QPushButton *>("pbOptSensorsApply")->setToolTip(tip);
    if(m_currentSelectedIndex > -1 && cp->minIsSet() && cp->maxIsSet())
    {
        m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setMinimum(cp->minValue());
        m_tabWidget->findChild<ENumeric *>("tdsbOptSensorsWeight")->setMaximum(cp->maxValue());
    }
}

void OptSensorsWeightManager::refreshLeds(const TVariant &data)
{
	QVector<double> v = data.toDoubleVector();
    int index = sender()->property("index").toInt();
    QColor c = KDARKGRAY;
    qDebug() << __FUNCTION__ << "vector bool" << v << "size" << v.size() << "index " << index;
    if(v.size() > 4)
    {
        int irb = static_cast<int> (v.at(1));
        int ioc = static_cast<int> (v.at(4));
        bool readback = irb & 0x0002;
        bool optimization_controlled = ioc & 0x0010;

        ELed *stReadbackLed = findChild<ELed*>(QString("lsr%1").arg(index));
        ELed *optControlledLed = findChild<ELed*>(QString("loc%1").arg(index));
        if(!stReadbackLed)
            perr("OptSensorsWeightManager::refreshLeds: readback led not found!");
        else
        {
	    stReadbackLed->setToolTip(data.message()+ " | index " + QString::number(index) );
	    stReadbackLed->setDisabled(data.quality() == Tango::ATTR_INVALID);
            readback ? c = KGREEN : c = KCAMEL;
            stReadbackLed->setColor(c);
        }
        if(!optControlledLed)
            perr("OptSensorsWeightManager::refreshLeds: optimization controlled led not found!");
        else
        {
		optControlledLed->setToolTip(data.message()+ " | index " + QString::number(index));
	    optControlledLed->setDisabled(data.quality() == Tango::ATTR_INVALID);

            optimization_controlled ? c = KGREEN : c = KCAMEL;
            optControlledLed->setColor(c);
        }


    }
}
