#include "weightmanager.h"
#include <QWidget>
#include <QGridLayout>
#include <QLabel>
#include <ENumeric>
#include <TSimpleLabel>
#include <ReadersGroup>
#include <SpectrumWriter>
#include <QPushButton>
#include <QTWatcher>
#include <QTWriter>
#include <macros.h>
#include "weightlabel.h"

WeightManager::WeightManager(QWidget *containerWidget) : QWidget(containerWidget)
{
    m_tabWidget = containerWidget;
    m_currentSelectedIndex = -1;
    connect(m_tabWidget->findChild<QPushButton *>("pbApply"), SIGNAL(clicked()), this, SLOT(apply()));
    m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setEnabled(false);
    m_tabWidget->findChild<QPushButton *>("pbApply")->setEnabled(false);
    QGridLayout *gridLo = new QGridLayout(this);
}

void WeightManager::setupBpms(const QStringList& labels, const QString &horBpmAttr, const QString &verBpmAttr)
{
    QGridLayout *lo = this->findChild<QGridLayout *>();
    QLabel *bpmHorLabel = new QLabel(this);
    bpmHorLabel->setText("Horizontal");
    bpmHorLabel->setAlignment(Qt::AlignHCenter);
    QLabel *bpmVerLabel = new QLabel(this);
    bpmVerLabel->setText("Vertical");
    bpmVerLabel->setAlignment(Qt::AlignHCenter);
    QLabel *corrHorLabel = new QLabel(this);
    corrHorLabel->setText("Horizontal");
    corrHorLabel->setAlignment(Qt::AlignHCenter);
    QLabel *corrVerLabel = new QLabel(this);
    corrVerLabel->setText("Vertical");
    corrVerLabel->setAlignment(Qt::AlignHCenter);
    QLabel *eigHorLabel = new QLabel(this);
    eigHorLabel->setText("Horizontal");
    eigHorLabel->setAlignment(Qt::AlignHCenter);
    QLabel *eigVerLabel = new QLabel(this);
    eigVerLabel->setText("Vertical");
    eigVerLabel->setAlignment(Qt::AlignHCenter);

    lo->addWidget(bpmHorLabel, 0, 1, 1, 1);
    lo->addWidget(bpmVerLabel, 0, 2, 1, 1);
    lo->addWidget(corrHorLabel, 0, 4, 1, 1);
    lo->addWidget(corrVerLabel, 0, 5, 1, 1);
    lo->addWidget(eigHorLabel, 0, 7, 1, 1);
    lo->addWidget(eigVerLabel, 0, 8, 1, 1);

    int row = 0;
    {
        ReadersGroup *rgHor = new ReadersGroup(this);
        foreach (QString label, labels)
        {
            QLabel *lab = new QLabel(this);
            lab->setText(label);
            WeightLabel *wl = new WeightLabel(this, label, row);
            wl->setProperty("associatedTangoPoint", horBpmAttr);
            wl->setObjectName("Horizontal " + label);
            connect(wl, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgHor->addReader(wl, row);

            row++;
            lo->addWidget(lab, row, BpmLabels, 1, 1);
            lo->addWidget(wl, row, HBpm, 1, 1);

        }
        rgHor->setSource(horBpmAttr);
    }

    row = 0;
    {
        ReadersGroup *rgVer = new ReadersGroup(this);
        foreach (QString label, labels)
        {
            WeightLabel *wl = new WeightLabel(this, label, row);
            wl->setObjectName("Vertical " + label);
            wl->setProperty("associatedTangoPoint", verBpmAttr);
            connect(wl, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgVer->addReader(wl, row);

            row++;
            lo->addWidget(wl, row, VBpm, 1, 1);
        }
        rgVer->setSource(verBpmAttr);
    }
}

void WeightManager::setupCorrAndEig(const QStringList &labels,
                                    const QString& horCorrAttr, const QString& verCorrAttr,
                                    const QString& horEigAttr, const QString& verEigAttr)
{
//    qDebug() << __FUNCTION__ << labels;
    QGridLayout *lo = this->findChild<QGridLayout *>();
    int row = 0;
    {
        ReadersGroup *rgHor = new ReadersGroup(this);
        foreach (QString label, labels)
        {
            QLabel *lab = new QLabel(this);
            lab->setText(label);
            WeightLabel *wl = new WeightLabel(this, label, row);
            wl->setProperty("associatedTangoPoint", horCorrAttr);
            wl->setObjectName("Horizontal " + label);
            connect(wl, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgHor->addReader(wl, row);

            row++;
            lo->addWidget(lab, row, CorrLabels, 1, 1);
            lo->addWidget(wl, row, HCorr, 1, 1);

        }
        rgHor->setSource(horCorrAttr);
    }

    row = 0;
    {
        ReadersGroup *rgVer = new ReadersGroup(this);
        foreach (QString label, labels)
        {
            WeightLabel *wl = new WeightLabel(this, label, row);
            wl->setObjectName("Vertical " + label);
            wl->setProperty("associatedTangoPoint", verCorrAttr);
            connect(wl, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgVer->addReader(wl, row);

            row++;
            lo->addWidget(wl, row, VCorr, 1, 1);
        }
        rgVer->setSource(verCorrAttr);
    }

    row = 0;
    {
        ReadersGroup *rgHEig = new ReadersGroup(this);
        ReadersGroup *rgVEig = new ReadersGroup(this);
        for(row = 0; row < labels.size(); row++)
        {
            QString label = QString("Eigenvalue %1").arg(row);
            QLabel *lab = new QLabel(this);
            lab->setText(QString::number(row));
            lab->setAlignment(Qt::AlignRight);
            WeightLabel *wlhe = new WeightLabel(this, label, row);
            wlhe->setObjectName("Horizontal " + label);
            wlhe->setProperty("associatedTangoPoint", horEigAttr);
            connect(wlhe, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgHEig->addReader(wlhe, row);
            lo->addWidget(lab, row + 1, EigLabels, 1, 1);
            lo->addWidget(wlhe, row + 1, HEig, 1, 1);
        }
        for(row = 0; row < labels.size(); row++)
        {
            QString label = QString("Eigenvalue %1").arg(row);
            WeightLabel *wlve = new WeightLabel(this, label, row);
            wlve->setObjectName("Vertical " + label);
            wlve->setProperty("associatedTangoPoint", verEigAttr);
            connect(wlve, SIGNAL(clicked(QString)), this, SLOT(manageSelection(QString)));
            rgVEig->addReader(wlve, row);

            lo->addWidget(wlve, row + 1, VEig, 1, 1);
        }
        rgHEig->setSource(horEigAttr);
        rgVEig->setSource(verEigAttr);
    }


}

void WeightManager::manageSelection(const QString &oName)
{
    m_currentSetPointVector.clear();
    m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setEnabled(false);
    m_tabWidget->findChild<QPushButton *>("pbApply")->setEnabled(false);

    foreach(WeightLabel *l, findChildren<WeightLabel *>())
    {
        l->setSelected(l->objectName() == oName);
        if(l->objectName() == oName)
            m_currentSelectedIndex = l->index();
    }
    /* label */
    m_tabWidget->findChild<TSimpleLabel *>("tslCurrentWeight")->setText(oName);
    QTWatcher *setPtWatcher = new QTWatcher(this);
    /* destroy watcher after first reading (qtango 4.3) */
    setPtWatcher->setAutoDestroy(true);
    connect(setPtWatcher, SIGNAL(attributeAutoConfigured(const TangoConfigurationParameters*)),
            this, SLOT(configurationReady(const TangoConfigurationParameters*)));

    connect(setPtWatcher->qtangoComHandle(), SIGNAL(newData(TVariant)), this, SLOT(initWriter(TVariant)));
    setPtWatcher->setSource(sender()->property("associatedTangoPoint").toString());
    m_tabWidget->findChild<QPushButton *>("pbApply")->setProperty("associatedTangoPoint",
                    sender()->property("associatedTangoPoint").toString());
}

void WeightManager::apply()
{
    QTWriter *writer = findChild<QTWriter *>();
    if(writer && m_currentSetPointVector.size() > m_currentSelectedIndex)
    {

        m_currentSetPointVector[m_currentSelectedIndex] = m_tabWidget->findChild<ENumeric *>("tdsbWeight")->value();
        QVariant argument(m_currentSetPointVector);
        qDebug() << __FUNCTION__ << m_currentSetPointVector << "execute" << writer->realTargets() <<
                    "index " << m_currentSelectedIndex;
        writer->QTangoComProxyWriter::execute(argument);
    }
}

void WeightManager::initWriter(const TVariant &v)
{
    if(m_currentSelectedIndex > -1)
    {
        /* false: get set point */
        QVector<double> setPtValues = v.toDoubleVector(false);
        QVector<double> readValues = v.toDoubleVector(true);
//       printf("\e[1;33msetPtValues sz %d readVal size %d curr index %d\e[0m\n",
//              setPtValues.size(), readValues.size(), m_currentSelectedIndex);
        if(setPtValues.size() == readValues.size() && setPtValues.size() > m_currentSelectedIndex)
        {
            m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setValue(setPtValues.at(m_currentSelectedIndex));
            foreach(double d, setPtValues)
                m_currentSetPointVector << d;
        }
        else
        {
            foreach(double d, readValues)
                m_currentSetPointVector << d;
            qDebug() << "uso read" << m_currentSetPointVector << v.tangoPoint();
            m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setValue(readValues.at(m_currentSelectedIndex));

        }
        QString currentTarget = m_tabWidget->findChild<QPushButton *>("pbApply")->property("associatedTangoPoint").toString();
        QTWriter *writer = findChild<QTWriter *>();

        if((writer != NULL && writer->targets() != currentTarget) || writer == NULL)
        {
            delete writer;
            writer = new QTWriter(this);
            connect(writer, SIGNAL(connectionOk(bool)), m_tabWidget->findChild<QPushButton *>("pbApply"),
                    SLOT(setEnabled(bool)));
            writer->setTargets(currentTarget);
        }
        if(writer && writer->qtangoComHandle()->configured())
        {
            m_tabWidget->findChild<QPushButton *>("pbApply")->setEnabled(true);
            m_tabWidget->findChild<QPushButton *>("pbApply")->
                    setToolTip(QString("%1").arg(currentTarget.split("/").last()));
        }
    }
}

void WeightManager::configurationReady(const TangoConfigurationParameters *cp)
{
    AttributeInfoEx aiex = cp->attributeInfoEx();
    m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setEnabled(true);
    QString tip = QString("%1\n%2").arg(QString::fromStdString(aiex.name)).arg(
                QString::fromStdString(aiex.description));
    m_tabWidget->findChild<QPushButton *>("pbApply")->setToolTip(tip);
    if(m_currentSelectedIndex > -1 && cp->minIsSet() && cp->maxIsSet())
    {
        m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setMinimum(cp->minValue());
        m_tabWidget->findChild<ENumeric *>("tdsbWeight")->setMaximum(cp->maxValue());
    }
}
